
var language = null;
var imageEnable = null;

function init()
{
  // Get arguments values of address bar
  var args = location.search.substr(1).split(/&/);

  var argsValues = [];

  for(var i =0; i < args.length; i++)
  {
    var tmp = args[i].split(/=/);
    if (tmp[0] != "")
    {
      argsValues[decodeURIComponent(tmp[0])] = decodeURIComponent(tmp.slice(1).join("").replace("+", " "));
    }
  }

  // Get params chose in the menu
  var baseURI = argsValues["selection"];
  language = argsValues["language"];
  imageEnable = argsValues["image_enable"];

  var config = new Config();
  config.load(function()
  {
    // Create the link filter
    var filterLink = "";
    for (key in config.edgesTypes)
    {
      filterLink += '?link = <'+config.linkPrefix + key + '> || ';
    }
    filterLink = filterLink.substring(0, filterLink.length-4);

    var endPoint = "https://query.wikidata.org/sparql";

    var nodes = new vis.DataSet([]);
    var edges = new vis.DataSet([]);

    var container = document.getElementById('mynetwork');

    // Init the Vis-Network
    var data = {
      nodes: nodes,
      edges: edges
    };
    var options = {
      interaction: {
        hover:true
      },
      nodes: {
        shape: "circularImage",
        borderWidth: 2,
        imagePadding : 7,
        size: 30,
        color: {
          border: "#cccccc",
          background: "#f0f0f0"
        },
        font: {
          color: "#222222",
          background : "#e0e0e0",
          bold: { color: "#000000" },
        },
        shapeProperties: {
          useBorderWithImage: true
        }
      },
      edges: {
        arrows: {
          to:     {enabled: true, scaleFactor:1, type:'arrow'},
          middle: {enabled: false, scaleFactor:1, type:'arrow'},
          from:   {enabled: false, scaleFactor:1, type:'arrow'}
        },
        font: {
          color: "#444444",
          strokeWidth: 2
        },
        color: {
          color: "lightgray",
          highlight: "lightgray",
          hover: "lightgray"
        },
        selectionWidth: 5,
        width: 3
      },
      physics: {
        forceAtlas2Based: {
        	gravitationalConstant: -15,
        	centralGravity: 0.001,
        	springLength: 150,
        	springConstant: 0.08
        },
        maxVelocity: 150,
        solver: "forceAtlas2Based",
        timestep: 0.35,
        stabilization: {
        	enabled: true,
        	iterations: 2000,
        	updateInterval: 25
        }
      }
    };

    // Initialize your network
    var network = new vis.Network(container, data, options);

    getData(baseURI, nodes, edges, config, filterLink, true);

    // Draw loading
    network.on("stabilizationProgress", function(params)
    {
      $("#loadingInfos").html("<h1> G�n�ration du graphe (" + parseInt((params.iterations / params.total) * 100)  + "%) </h1>");
    });
    network.on("stabilizationIterationsDone", function(params)
    {
      $("#loadingInfos").css("display", "none");
    });

    // Double click in nade = open new nav tab with wikidata URI
    network.on("doubleClick", function(data)
    {
      if(data.nodes.length > 0)
      {
        if(!nodes.get(data.nodes[0]).edgesLoaded)
        {
          getData(data.nodes[0], nodes, edges, config, filterLink, false);
        }
      }
    });

    // Click simple
    network.on("click", function(data)
    {
      $("#rightUI").html("");

      if(data.nodes.length > 0)
      {
        var node = nodes.get(data.nodes[0]);

        UpdateRightUI(node, config);
      }
    });
  });
}

/**
 * Add a new node in the graph
 * @param {String}           uri                 URI of the node
 * @param {String}           label               Label of the node
 * @param {String}           image               Image URL
 * @param {Arrat}            nodeDesign          Params of the nodes design
 * @param {Vis.DataSet}      nodes               Dataset of all nodes
 * @param {Boolean}          edgesLoaded         Indicate if linked data of this node is loaded (for storage)
 */
function addNode(uri, label, imageUrl, nodeDesign, nodes, edgesLoaded)
{
  var color = "#999999";

  var size = nodeDesign["size"];

  var backgroundColor = nodeDesign["backgroundColor"];
  var borderColor = nodeDesign["borderColor"];
  var image = nodeDesign["image"];
  var shape = "circularImage";
  var imagePadding = 7;

  // Get image if images is enable
  if(imageUrl && imageEnable)
  {
    image = imageUrl;
    shape = "image";
    imagePadding = 0;
  }

  var borderSize = 2;
  if(edgesLoaded)
  {
    borderSize = 5;
  }

  nodes.add([
    {
      shape: shape,
      id: uri,
      uri : uri,
      label: label,
      image: image,
      size : size,
      margin : 15,
      imagePadding : imagePadding,
      shapeProperties : {
        size : 120,
        interpolation: true
      },
      borderWidth : borderSize,
      borderWidthSelected: borderSize,
      edgesLoaded : edgesLoaded,
      color: {
        background: backgroundColor,
        border: borderColor,
        highlight: {
          border: "#000000",
          background: "#ffffff"
        }
      }
    }
  ]);
}

/**
 * Add a new edge in the graph
 * @param {String}           uriEl1          URI of the first node
 * @param {String}           uriEl2          URI of the second node
 * @param {Array}            edgeDesign      Design of the edge
 * @param {String}           key             Key of the edge
 * @param {Vis.DataSet}      nodes           Dataset of all nodes
 * @param {Vis.DataSet}      edges           Dataset of all edges
 */
function addEdge(uriEl1, uriEl2, edgeDesign, key, nodes, edges)
{
  // Get linked nodes ids
  var idEl1 = nodes.getIds({
    filter: function(item) {
      return item.uri == uriEl1;
    }
  })[0];

  var idEl2 = nodes.getIds({
    filter: function(item) {
      return item.uri == uriEl2;
    }
  })[0];

  // Get design informations
  var color = edgeDesign["color"];
  var label = edgeDesign["label"];

  // Create background color of the label
  var r = parseInt(color.substring(1, 3), 16) + 90;
  var g = parseInt(color.substring(3, 5), 16) + 90;
  var b = parseInt(color.substring(5, 7), 16) + 90;

  if(r > 235)
  {
    r = 235;
  }
  if(g > 235)
  {
    g = 235;
  }
  if(b > 235)
  {
    b = 235;
  }
  var fontBackgoundColor = "#" + r.toString(16) + g.toString(16) + b.toString(16);

  // Create Nodes
  edges.add([
    {
      from: idEl1,
      to: idEl2,
      label: label,
      color: {
        color: color,
        highlight: color,
        hover: color
      },
      uriSrc : uriEl1,
      uriDest : uriEl2,
      key : key,
      font: {
        color: "#333333",
        size: 12,
        strokeColor: fontBackgoundColor
      }
    }
  ]);
}

/**
 * Check if node already exist
 * @param {String}           uri            URI of the node
 * @param {Vis.DataSet}      nodes          Dataset of all nodes
 */
function nodeAlreadyExist(uri, nodes)
{
  var nodeList = nodes.get({
    filter: function(item) {
      return item.uri == uri;
    }
  });

  if(nodeList.length > 0)
  {
    return true;
  }
  else
  {
    return false;
  }
}

/**
 * Check if edge already exist
 * @param {String}           uriSrc           URI of the first linked node
 * @param {String}           uriDest          URI of the second linked node
 * @param {String}           key              Key of the edge
 * @param {Vis.DataSet}      edges            Dataset of all edges
 */
function edgeAlreadyExist(uriSrc, uriDest, key, edges)
{
  var edgeList = edges.get({
    filter: function(item) {
      return item.uriSrc == uriSrc && item.uriDest == uriDest && item.key == key;
    }
  });

  if(edgeList.length > 0)
  {
    return true;
  }
  else
  {
    return false;
  }
}

/**
 * Load the config file
 * @param {String}      callback           The callback
 */
function loadConfig(callback)
{
  var fileName = "config.json";

  var jqxhr = $.getJSON(fileName, null)
  .done(function(content)
  {
    callback(content);
  })
  .fail(function(d, textStatus, error)
  {
    console.error("getJSON failed, status: " + textStatus + ", error: "+error);
  })
  .always(function()
  {

  });
};

/**
 * Create request for get linked data from an URI and update nodes and edges datas
 * @param {String}              nodeUri              Uri of the main node
 * @param {Vis.DataSet}         nodes                Dataset of all nodes
 * @param {Vis.DataSet}         edges                Dataset of all edges
 * @param {Array}               config               Array of config data
 * @param {String}              filterLink           SPARQL Filter of data types
 * @param {Boolean}             firstLoading         True if is the first loading of data (initialisation)
 */
function getData(nodeUri, nodes, edges, config, filterLink, firstLoading)
{
  $("#loadingInfos").html("<h3> Rcupration des donnes </h3>");

  // Create Request
  var request = '';
  request += 'SELECT (<'+nodeUri+'> as ?element) (group_concat(distinct ?elementType;separator=";") as ?elementTypes) ?elementLabel (group_concat(distinct ?link;separator=";") as ?links) ?linkedEl ?linkedLabel (group_concat(distinct ?linkedType;separator=";") as ?linkedTypes) ?invert ?image ?linkedImage ';
  request += 'WHERE ';
  request += '{ ';
  request += '  { ';
  request += '    <'+nodeUri+'> rdfs:label ?elementLabel . ';
  request += '    <'+nodeUri+'> wdt:P31 ?elementType . ';
  request += '    <'+nodeUri+'> ?link ?linkedEl . ';
  request += '    ?linkedEl rdfs:label ?linkedLabel . ';
  request += '    ?linkedEl wdt:P31 ?linkedType . ';
  request += '    OPTIONAL { <'+nodeUri+'> wdt:P18 ?image } . ';
  request += '    OPTIONAL { ?linkedEl wdt:P18 ?linkedImage } . ';
  request += '    BIND(false AS ?invert) . ';
  request += '    FILTER(LANG(?elementLabel) = "'+language+'" ) . ';
  request += '    FILTER(LANG(?linkedLabel) = "'+language+'" ) . ';
  request += '    FILTER('+filterLink+') . ';
  request += '  } ';
  request += '  UNION { ';
  request += '    <'+nodeUri+'> rdfs:label ?elementLabel . ';
  request += '    <'+nodeUri+'> wdt:P31 ?elementType . ';
  request += '     ?linkedEl ?link <'+nodeUri+'> . ';
  request += '    ?linkedEl rdfs:label ?linkedLabel . ';
  request += '    ?linkedEl wdt:P31 ?linkedType . ';
  request += '    OPTIONAL { <'+nodeUri+'> wdt:P18 ?image } . ';
  request += '    OPTIONAL { ?linkedEl wdt:P18 ?linkedImage } . ';
  request += '    BIND(true AS ?invert) . ';
  request += '    FILTER(LANG(?elementLabel) = "'+language+'" ) . ';
  request += '    FILTER(LANG(?linkedLabel) = "'+language+'" ) . ';
  request += '    FILTER('+filterLink+') . ';
  request += '  } ';
  request += '} ';
  request += 'GROUP BY ?element ?elementType ?elementLabel ?linkedEl ?linkedLabel ?invert ?image ?linkedImage ';

  var queryURL = config.endPoint + "?" + "query="+escape(request) + "&format=json";

  // launch the query
  var ajaxRequest = $.ajax({
    url:queryURL,
    dataType: 'json'
  });

  ajaxRequest.fail(function()
  {
    // if the request fails, return to menu
    console.log("EndPoint : " + endPoint);
    console.log("Requete : " + request);
    alert("Echec de la rcupration des donnes");
    $("#loadingInfos").html("");
  });

  ajaxRequest.done(function(data)
  {
    $("#loadingInfos").html("");

    if(!firstLoading && data.results.bindings.length > config.nodeLimit)
    {
      if(confirm(data.results.bindings.length + " noeuds rcuprs, cela risque de ralentir votre ordinateur, voulez vous les afficher quand mme"))
      {
        addData(data, nodes, edges, config, filterLink, firstLoading);
      }
    }
    else
    {
      addData(data, nodes, edges, config, filterLink, firstLoading);
    }
  });
};

/**
 Create Datas (nodes and edges)
 * @param {Array[]}             data                 Array of all datas get from request
 * @param {Vis.DataSet}         nodes                Dataset of all nodes
 * @param {Vis.DataSet}         edges                Dataset of all edges
 * @param {Array}               config               Array of config data
 * @param {String}              filterLink           SPARQL Filter of data types
 * @param {Boolean}             firstLoading         True if is the first loading of data (initialisation)
 */
function addData(data, nodes, edges, config, filterLink, firstLoading)
{
  for(var i = 0; i < data.results.bindings.length; i++)
  {
    var elementUri = data.results.bindings[i]["element"]["value"];
    if(firstLoading)
    {
      // Create the main Element
      var elementLabel = data.results.bindings[i]["elementLabel"]["value"];
      var elementTypes = data.results.bindings[i]["elementTypes"]["value"].split(";");
      var image = "";
      if(data.results.bindings[i]["image"] != undefined)
      {
        image = data.results.bindings[i]["image"]["value"];
      }

      if(!nodeAlreadyExist(elementUri, nodes))
      {
        var nodeDesign = null;
        // Get design param in nodesByEdgesTypes
        for (key in config.nodesTypes)
        {
          if(elementTypes.includes(config.nodePrefix + key))
          {
            nodeDesign = config.nodesTypes[key];
          }
        }

        // Get defaut params
        if(nodeDesign == null)
        {
          nodeDesign = config.nodesTypes["*"];
        }

        addNode(elementUri, elementLabel, image, nodeDesign, nodes, true);
      }
    }
    else
    {
      // Update drawing element for indicate is edges are loaded
      nodes.update({id: elementUri, borderWidth: 5, edgesLoaded: true});
    }

    // Create the linked nodes
    if(data.results.bindings[i]["linkedEl"] != undefined)
    {
      var linkedElementUri = data.results.bindings[i]["linkedEl"]["value"];
      var linkedElementLabel = data.results.bindings[i]["linkedLabel"]["value"];
      var linkedElementTypes = data.results.bindings[i]["linkedTypes"]["value"].split(";");
      var links = data.results.bindings[i]["links"]["value"].split(";");
      var invert = data.results.bindings[i]["invert"]["value"];
      var linkedImage = "";
      if(data.results.bindings[i]["linkedImage"] != undefined)
      {
        var linkedImage = data.results.bindings[i]["linkedImage"]["value"];
      }


      if(!nodeAlreadyExist(linkedElementUri, nodes))
      {
        var nodeDesign = null;

        // Get design param in nodesByEdgesTypes
        for (key in config.nodesTypes)
        {
          if(linkedElementTypes.includes(config.nodePrefix + key))
          {
            nodeDesign = config.nodesTypes[key];
          }
        }

        if(nodeDesign == null)
        {
          // Get design param in nodesByEdgesTypes
          for (key in config.nodesByEdgesTypes)
          {
            if(links.includes(config.linkPrefix + key))
            {
              nodeDesign = config.nodesByEdgesTypes[key];
            }
          }

          // Get defaut params
          if(nodeDesign == null)
          {
            nodeDesign = config.nodesTypes["*"];
          }
        }

        addNode(linkedElementUri, linkedElementLabel, linkedImage, nodeDesign, nodes, false);
      }


      // Create the relations edges
      for(var j = 0; j < links.length; j++)
      {
        var el1Uri = elementUri;
        var el2Uri = linkedElementUri;
        if(invert == "true")
        {
          el1Uri = linkedElementUri;
          el2Uri = elementUri;
        }

        if(!edgeAlreadyExist(el1Uri, el2Uri, links[j], edges))
        {
          // Get design param in config
          var edgeDesign = null;
          for (key in config.edgesTypes)
          {
            if(config.linkPrefix + key == links[j])
            {
              edgeDesign = config.edgesTypes[key];
            }
          }
          if(edgeDesign == null)
          {
            edgeDesign = config.edgesTypes["*"];
            edgeDesign["label"] = links[j];
          }

          addEdge(el1Uri, el2Uri, edgeDesign, links[j], nodes, edges);
        }
      }
    }
  }
}

/**
 Update right UI with node informations (request linked element of the URO)
 * @param {Vis.Node}             node                 Informations of the node
 * @param {Config}               config               Config informations
 */
function UpdateRightUI(node, config)
{
  request = "SELECT ?predicate ?object ?language ";
  request += "WHERE {";
  request += " <"+node.id+"> ?predicate ?object ";
  request += "}";

  var queryURL = config.endPoint + "?" + "query="+escape(request) + "&format=json";

  // launch the query
  var ajaxRequest = $.ajax({
    url:queryURL,
    dataType: 'json'
  });

  ajaxRequest.fail(function()
  {
    // if the request fails, return to menu
    console.log("EndPoint : " + endPoint);
    console.log("Requete : " + request);
    $("#rightUI").html("Echec de la rcupration des donnes");
  });

  ajaxRequest.done(function(data)
  {
    var content = "";
    content += "<h3>"+node.id+"</h3>";

    for(var i = 0; i < data.results.bindings.length; i++)
    {
      var lang = "";
      if(data.results.bindings[i]["object"]["xml:lang"] != undefined)
      {
        lang = data.results.bindings[i]["object"]["xml:lang"];
      }

      if(lang == "" || lang == language)
      {
        var predicate = data.results.bindings[i]["predicate"]["value"];
        var object = data.results.bindings[i]["object"]["value"];

        if(predicate == "http://www.w3.org/2000/01/rdf-schema#label" || predicate == "http://schema.org/description" || predicate == "http://www.w3.org/2004/02/skos/core#altLabel")
        {
          if(predicate == "http://www.w3.org/2000/01/rdf-schema#label")
          {
            predicate = "Label"
          }
          else if(predicate == "http://schema.org/description")
          {
            predicate = "Description"
          }
          else if(predicate == "http://www.w3.org/2004/02/skos/core#altLabel")
          {
            predicate = "Label secondaire"
          }
          content += "<p>" + predicate + " : <b>" + object + "</b></p>";
        }
      }
    }

    $("#rightUI").html(content);
  });
}
